#!/usr/bin/env python3

import sys

sys.path.append("lib")

from charms.osm.ns import NetworkService

from ops.charm import CharmBase
from ops.main import main
from ops.model import (
    ActiveStatus,
    BlockedStatus,
    MaintenanceStatus,
    WaitingStatus,
    ModelError,
)
import json
import os
import subprocess

def install_dependencies():
    # Make sure Python3 + PIP are available
    if not os.path.exists("/usr/bin/python3") or not os.path.exists("/usr/bin/pip3"):
        # This is needed when running as a k8s charm, as the ubuntu:latest
        # image doesn't include either package.

        # Update the apt cache
        subprocess.check_call(["apt-get", "update"])

        # Install the Python3 package
        subprocess.check_call(["apt-get", "install", "-y", "python3", "python3-pip"],)

    REQUIREMENTS_TXT = "{}/requirements.txt".format(os.environ["JUJU_CHARM_DIR"])
    if os.path.exists(REQUIREMENTS_TXT):
        subprocess.check_call(
            [sys.executable, "-m", "pip", "install", "-r", REQUIREMENTS_TXT],
        )


install_dependencies()


class NetworkServiceCharm(CharmBase):
    def __init__(self, *args):
        super().__init__(*args)

        # Register all of the events we want to observe
        for event in (
            # Charm events
            self.on.config_changed,
            self.on.install,
            self.on.upgrade_charm,
            # Charm actions (primitives)
            self.on.touch_action,
        ):
            self.framework.observe(event, self)

    def on_config_changed(self, event):
        """Handle changes in configuration"""
        unit = self.model.unit

        # Unit should go into a waiting state until Juju credentials are provided
        unit.status = WaitingStatus("Waiting for Juju credentials")

        # Check that we have a juju username/password
        if self.model.config["juju-username"] and self.model.config["juju-password"]:
            unit.status = ActiveStatus()
        else:
            unit.status = BlockedStatus("Invalid Juju credentials.")

    def on_install(self, event):
        """Called when the charm is being installed"""
        # unit = self.model.unit

        # unit.status = ActiveStatus()
        pass

    def on_upgrade_charm(self, event):
        """Upgrade the charm."""
        unit = self.model.unit

        # Mark the unit as under Maintenance.
        unit.status = MaintenanceStatus("Upgrading charm")

        self.on_install(event)

        # When maintenance is done, return to an Active state
        unit.status = ActiveStatus()

    ####################
    # NS Charm methods #
    ####################

    def on_touch_action(self, event):
        filename = event.params["filename"]
        client = NetworkService(
            user=self.model.config["juju-username"],
            secret=self.model.config["juju-password"],
        )
        ns_config_info = json.loads(self.model.config["ns_config_info"])
        results = []
        for key in ns_config_info["osm-config-mapping"]:
            application = ns_config_info["osm-config-mapping"][key]
            #     "{}.{}.{}".format(
            #         "1", "mgmtVM", "0",  # member-vnf-index  # vdu_id  # vdu_count_index
            #     )
            # ]
            result = self.touch_files(client, application, filename)
            results.append(result)
        event.set_results(results)

    def touch_files(self, client, application, filename):
        """Touch files to all VNFs"""
        output = client.ExecutePrimitiveGetOutput(
            application, "touch", params={"filename": filename}, timeout=500,
        )
        return output


if __name__ == "__main__":
    main(NetworkServiceCharm)
